
//*****************************************************************************
//
// File Name	: 'SED1530.c'
// Title		: SPI interface driver for the E0855-2 SED1530 graphic LCD
// Author		: Marc Meise
// Created		: December 2007
// Version		: Beta
// Target MCU	: NXP LPC210x
//
//
// This code is distributed under the GNU Public License
// which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

/*  
	Disclaimer:
	This code is highly beta and might lack some funcionality and/or documentation.
	Furthermore the code has not been properly tested.
	Nevertheless, please expect most functions to work.
	
	01.12.2008 Fixed a bug which affected the "end of line"/newline determination.
			   Thanks to Klaus W. from mikrocontroller.net
	21.01.2008 Added function "SED_7segment"
	06.12.2007 Ported the code from AVR-GCC to ARM-ELF-GCC
	05.12.2007 Added support for software-SPI
	05.12.2007 Added function SED_pixel as a (very) basic graphic routine
	30.11.2007 Made CS and /CS redundant by switch "CS_ON"
	30.11.2007 Added "SED_contrast" function
	29.11.2007 Replaced the font files which were modified and debuged by Werner Boellmann


	I will not take any responsibility or what so ever for the functionality of the E0855-2 SED1530 driver.
	If you wreck anything - it's your own business...
	Modify this code and contribute freely.
*/

#include "lpc2103.h"
#include <inttypes.h>
#include "font.h"
#include <string.h>
#include "PICS.h"

// Port-definitions
// Alter the Port-definitions to your convenience.

//#define LCDPORT		IOSET			//PORT where the GLCD ist connected
#define A0			7					//A0
#define RES 		5					//Reset
#define CS1 		4					//CS1 is Low-active
#define CS2 		6					//CS2 is high-active
#define MOSI		2					//MOSI
#define SCK			8					//SCK
#define CS_ON       1					//If CS and /CS are hardwired (not connected to the Controller, then set to '0')
#define	SPISCALE    16					//SPI-Prescaler

//Software-SPI

const unsigned char	SOFT_SPI=1;		// Use Software SPI (1=enable; 0=disable software-SPI)
#define SOFT_MOSI   	2				// Portpin MOSI for software-SPI
#define SOFT_SCK    	8				// Portpin SCK for software-SPI


// Only alter the following code, if you know what you're doing...

#define COL_Hi		0x11				//Column 16 HI is left for E0855-2			
#define COL_Lo		0x00				//Column 00 Lo is left for E0855-2
#define SC_width	96					//Screenwidth (96 for the E0855-2)

#define A0_Hi		IOSET=(1<<A0)		//Change A0 SED1530 command-mode
#define A0_Lo	  	IOCLR=(1<<A0)		//Change A0 SED1530 data-mode
#define RES_On  	IOCLR=(1<<RES)		//Reset SED1530
#define RES_Off  	IOSET=(1<<RES)		//Set SED1530


// Function Prototypes
void delay (void);
void Init_SPI (void);
void SPI_out (uint8_t Data);
void Init_SED1530 (void);
void SED_clear (void);
void SED_clear_page (uint8_t Page);
void SED_Chg_page (uint8_t Page);
void SED_Home_Col (void);
void Goto_XY (uint8_t X, uint8_t Y);
void SED_putc (char character);
void SED_puts (char *string);
void SED_symbol(uint8_t symbol, uint8_t OnOff);
void SED_7segment(uint8_t segno, uint8_t number);
void SED_newline(void);
void SED_showPIC (uint8_t picture);
void SED_puts_num (uint16_t numeric);
unsigned int itoa( int z, char* Buffer );
uint8_t ReverseBitorder (uint8_t Bits);
void SED_set_XY(uint8_t Page, uint8_t Column);
void SED_sleep (void);
void SED_wakeup (void);
void SED_contrast (uint8_t contrast);
void SED_pixel(uint8_t X, uint8_t Y, uint8_t pixelbyte);

//some Variables
volatile uint8_t data;							
volatile unsigned int pagecounter=0, columncounter=0;

//a little delay
void delay (void)
{
 volatile unsigned char a,b;
 for (a=0;a<=160;a++)
 	{
	 for (b=0;b<=150;b++)
	 	{
		 asm volatile("NOP");
	 	}
	 }
}


//Initialize SPI
void Init_SPI (void)				
{ 
 if(SOFT_SPI!=1)
 	{
 	 delay();									// wait a little
 	 PINSEL0 |= (1<<8)|(1<<10)|(1<<12)|(1<<14);
 	 S0SPCCR = SPISCALE; 						
	 S0SPCR = (1<<5); 							//Master-mode
	}
	else
		{
		 IODIR|=(1<<SOFT_MOSI); 
		 IODIR|=(1<<SOFT_SCK);
		 IOCLR=(1<<SOFT_MOSI); 
		 IOCLR=(1<<SOFT_SCK);
		}
}

//send one byte over SPI
void SPI_out (uint8_t Data)				//Hardware SPI-out
{
 if(SOFT_SPI!=1)
	{
	S0SPDR = Data; 							//Write Data
	while (!(S0SPSR & (1<<7)))			//Wait for EOT
	; 
	} 
	else	
		{									//Software SPI-out
		unsigned char out;
		IOCLR=(1<<SOFT_MOSI);
		IOCLR=(1<<SOFT_SCK);
		for(out=8; out>=1; out--)
			{
			IOCLR=(1<<SOFT_MOSI);  
			if(Data & 0x80)
				{
				IOSET=(1<<SOFT_MOSI); 
				} 
			Data = Data<<1;
			IOSET=(1<<SOFT_SCK);
			IOCLR=(1<<SOFT_SCK); 
			}
		}
}

//Initialize the SED1530 (E0855-2)
void Init_SED1530 (void)
{
 Init_SPI();								//Initialise the SPI
 RES_On;									// RESET the SED
 if (CS_ON>=1)
 	{
	 IOCLR=(1<<CS1);						// /CS1 goes LOW
	 IOSET=(1<<CS2);						// CS2 goes HI
	}
 RES_Off;									// SED is resetted
 delay();
 A0_Lo;										// Enter SED1530 command-mode
 SPI_out(0x40);								// Start display Line 0
 SPI_out(0xA1);								// ADC inverse direction
 SPI_out(0xA2);								// LCD-Bias= 1/5
 SPI_out(0xC0);								// Output Status 0
 SPI_out(0x2F);								// GLCD Voltage-Regulators ON
 delay();
 SPI_out(0x8F);								// Contrast=Medium
 SPI_out(0xA4);								// Display-Test OFF
 SPI_out(0xAF);								// Display ON
 SPI_out(0xA6);								// Display NOT inverted
 SPI_out(0xB3);								// Set Page 3						
 SPI_out(COL_Hi);							// Set Column by Hi 4-Bits															
 SPI_out(COL_Lo);							// Set Column by Low 4-Bits
 A0_Hi;										// Enter SED1530 data-mode										 
}

// Clear Screen
void SED_clear (void)
{
 uint8_t pagecount;
 for (pagecount=0; pagecount<=8; pagecount++)		//Clear Display by erasing all Pixels
 	{
	 SED_Chg_page(pagecount);						//Set appropriate Page
	 SED_Home_Col();								//Set left column
	 for(columncounter=0;columncounter<SC_width;columncounter++)
	 	{
		 SPI_out(0x00);
		}
	}
 SED_Chg_page(0x03);								//Change to Page 3
 SED_Home_Col();									//Set left column
}

// Clear Page
void SED_clear_page (uint8_t Page)
{
 SED_Chg_page(Page);								//Set appropriate Page
 SED_Home_Col();									//Set left column
 for (columncounter=0;columncounter<SC_width;columncounter++)
 	{
	 SPI_out(0x00);
	}
 SED_Home_Col();									//Set left column
}

//Change Page
void SED_Chg_page (uint8_t Page)
{
 A0_Lo;
 switch (Page)
	{
	 case (0) : SPI_out(0xB0); break;			//Set Page 0
	 case (1) : SPI_out(0xB1); break;			//Set Page 1
	 case (2) : SPI_out(0xB2); break;			//Set Page 2
	 case (3) : SPI_out(0xB3); break;			//Set Page 3
	 case (4) : SPI_out(0xB4); break;			//Set Page 4
	 case (5) : SPI_out(0xB5); break;			//Set Page 5
	 case (6) : SPI_out(0xB6); break;			//Set Page 6
	 case (7) : SPI_out(0xB7); break;			//Set Page 7
	 case (8) : SPI_out(0xB8); break;			//Set Page 8 (Special characters for The E0855-2)
	}
 pagecounter=Page;
 A0_Hi;
}


//Set the Left visible Corner of the Display
void SED_Home_Col (void)
{
 A0_Lo;											// Enter SED1530 command-mode
 SPI_out(COL_Hi);								// Set Column by Hi and Low 4-Bits
 SPI_out(COL_Lo);
 columncounter=0;
 A0_Hi;											// Enter SED1530 data-mode
}

//Display one Character
void SED_putc (char character)
{
 uint8_t column;
 if (character>31)
 	{
	 SPI_out(0x00);
	 columncounter++;
 	 for(column=0;column<=4;column++)
 		{
		 if (columncounter>=SC_width)
			{
			 SED_newline();	
			}
		 if (pagecounter<0)
			{
			 pagecounter=3;
			 SED_Chg_page(pagecounter);
			 SED_Home_Col ();	
			}	 	
 	 	 data=(font[character-32][column]);
 	 	 SPI_out(data);
		 columncounter++;
		}	
	}
}

//reverse the Bit-order for inverse Display of the E0855-2
//This is the only function, which is not my coding.
//Respect and honor to Mr. Huslik for this great function. 
//Not needed for displaying characters but still needed for graphics
uint8_t ReverseBitorder (uint8_t Bits)
{
  uint8_t ReversedBit;
  ReversedBit = (Bits >> 4) | (Bits << 4);
  ReversedBit = ((ReversedBit & 0xcc) >> 2 ) | ((ReversedBit & 0x33) << 2);
  ReversedBit = ((ReversedBit & 0xaa) >> 1 ) | ((ReversedBit & 0x55) << 1);
  return ReversedBit;
}

//display character-String
void SED_puts (char *string)
{
 while (*string)
 {
  SED_putc(*string);
  string++;
 }
}

//display numeric-Values
void SED_puts_num (uint16_t numeric)
{
 int buffer[20];
 itoa(numeric,buffer);
 SED_puts(buffer);
}

//itoa-function (taken from www.mikrocontroller.net/articles/FAQ#itoa.28.29)
//since WINARM does not support the iota-function (yet)
unsigned int itoa( int z, char* Buffer )
{
  int i = 0;
  int j;
  char tmp;
  if( z == 0 ) {
      Buffer[0] = '0';
      Buffer[1] = '\0';
  }
  else 
	{
     if( z < 0 ) 
		{
		 Buffer[0] = '-';
		 z = -z;
		 Buffer++;
		}
     while( z > 0 )
		{
		 Buffer[i++] = '0' + z % 10;
		 z /= 10;
		}
    for( j = 0; j < i / 2; ++j ) 
		{
		 tmp = Buffer[j];
		 Buffer[j] = Buffer[i-j-1];
		 Buffer[i-j-1] = tmp;
		} 
		Buffer[i] = '\0';		
	}
return Buffer;
}

//Show a picture out of the picture-file
void SED_showPIC (uint8_t picture)
{ 
 uint16_t column;
 SED_Chg_page(3);
 SED_Home_Col();
  	 for(column=0;column<=383;column++)
 		{
		 switch (picture)
 			{
	 		case (0): data=picture0[column]; break;	
	 		case (1): data=picture1[column]; break;
	 		case (2): data=picture2[column]; break;
	 		case (3): data=picture3[column]; break;
	 		case (4): data=picture4[column]; break;
	 		case (5): data=picture5[column]; break;
	 		case (6): data=picture6[column]; break;
	 		case (7): data=picture7[column]; break;
	 		case (8): data=picture8[column]; break;
	 		case (9): data=picture9[column]; break;
			}		 
		 data= ReverseBitorder(data);
 	 	 SPI_out(data);
		 switch (column)
		 	{
			 case (SC_width-1) : SED_Chg_page(2); SED_Home_Col(); break;
			 case ((SC_width-1)+SC_width): SED_Chg_page(1); SED_Home_Col(); break;
			 case ((SC_width-1)+(SC_width*2)): SED_Chg_page(0); SED_Home_Col(); break;
			}
		}
 SED_Chg_page(3);
 SED_Home_Col();	  
}

//Access to 7-segment-display (segno: most right=0, most left=3; number: 0 to 9)
void SED_7segment(uint8_t segno, uint8_t number)
{
 unsigned char temp,seg;
 switch (segno)								//Chose the desired 7-segment
 	{
	 case (0): segno=94; break;
	 case (1): segno=87; break;
	 case (2): segno=77; break;
	 case (3): segno=70; break;
	}
 seg=(segment[number]);
 for (temp=0; temp<=6; temp++)
 	{
	 if(seg & 0x80)								//Compare last bit
	 	{
	 	 SED_symbol(segno-temp,1);				//On match display
		}
	    else
			{
	 	 	 SED_symbol(segno-temp,0);			//No match, no display
			}
     seg<<=1;									//Shift left
	}
}

//Access to the Symboles of the E2855-2
void SED_symbol(uint8_t symbol, uint8_t OnOff)
{
 unsigned char temp;
 temp=pagecounter;
 SED_set_XY(8,symbol-16);
 if (OnOff>=1)
 	{
	 SPI_out(0x01);
    }
	else
		{
		 SPI_out(0x00);
		}
 SED_Chg_page(temp);
 SED_Home_Col();
 }

//Newline and carriage return
void SED_newline()
{
 pagecounter--;
 	if (pagecounter<0)
	{
	 pagecounter=3;
	 SED_Chg_page(pagecounter);
	 SED_Home_Col ();	
	}
	else
	{
 	 SED_Chg_page(pagecounter);
 	 SED_Home_Col ();
	}
}

//Select Page and Column
void SED_set_XY(uint8_t Page, uint8_t Column)
{
 SED_Home_Col ();
 SED_Chg_page (Page);
 Column=Column+16;
 volatile unsigned char col_HI=Column,col_LO=Column;
 columncounter=Column-16;
 A0_Lo;										
 col_HI=col_HI>>4;
 col_LO=col_LO<<4;
 col_LO=col_LO>>4;
 col_HI=col_HI+16;
 SPI_out(col_HI);
 SPI_out(col_LO);
 A0_Hi;
}

//Send Display to sleep
void SED_sleep (void)
{
 A0_Lo;
 SPI_out(0xAD);							//Static indicator ON
 SPI_out(0x28);							//Turn OFF coltage follower, regulator and booster
 A0_Hi;
}

// wake-up Display from sleep
void SED_wakeup (void)
{
 A0_Lo;
 SPI_out(0xAD);							//Static indicator OFF
 SPI_out(0x2F);							//Turn ON coltage follower, regulator and booster
 delay();
 A0_Hi;
}

//Set or change the contrast of the display in 32 (0-31) steps
void SED_contrast (uint8_t contrast)
{ 
 A0_Lo;
 if (contrast<=31)
 	{
	 contrast=contrast+128;
 	 SPI_out(contrast);					// Range is: 0 to 31
	}
	else
		{
		 SPI_out(0x8F);					// Contrast=Medium=15
		}
 A0_Hi;
}

//Set or unset a Pixel (Parameters: column[0-95], Page[0-3], Bytepattern)
void SED_pixel(uint8_t X, uint8_t Y, uint8_t pixelbyte)
{
 SED_set_XY(Y,X);						//SED_set_XY takes (Page/Column)
 SPI_out(pixelbyte);					//Send the vertical byte-pattern to the according Page
}

